@@ -1,7 +1,7 @@
 from matrix.util import indent
 
 from . import _dispatch
-from .util import register_module
+from .util import register_module, is_framework_module
 
 __all__ = ["Chain"]
 
@@ -11,15 +11,18 @@ class Chain:
     """A chain of links.
 
     Args:
-        *links (tuple): Links of the chain.
+        *links (object): Links of the chain.
+
+    Attributes:
+        links (tuple): Links of the chain.
     """
 
     def __init__(self, *links):
         # Filter `None`s.
         links = tuple(filter(None, links))
-        try:
+        if any(is_framework_module(link) for link in links):
             self.links = self.nn.ModuleList(links)
-        except Exception:
+        else:
             self.links = links
 
     def __call__(self, x):
@@ -27,12 +30,15 @@ class Chain:
             x = link(x)
         return x
 
-    def __iter__(self):
-        return iter(self.links)
-
     def __getitem__(self, item):
         return self.links[item]
 
+    def __len__(self):
+        return len(self.links)
+
+    def __iter__(self):
+        return iter(self.links)
+
     def __str__(self):
         return repr(self)
 
@@ -49,3 +55,17 @@ def code(chain: Chain, xz, z, x, **kw_args):
     for link in chain:
         xz, z = code(link, xz, z, x, **kw_args)
     return xz, z
+
+
+@_dispatch
+def code_track(chain: Chain, xz, z, x, h, **kw_args):
+    for link in chain:
+        xz, z, h = code_track(link, xz, z, x, h, **kw_args)
+    return xz, z, h
+
+
+@_dispatch
+def recode(chain: Chain, xz, z, h, **kw_args):
+    for link in chain:
+        xz, z, h = recode(link, xz, z, h, **kw_args)
+    return xz, z, h
